import os

from django.db import models
from django.utils.translation import gettext_lazy as _

from irekua_database.models import ItemType
from irekua_database.models import Visualizer


def visualizer_path(instance, filename):
    _, ext = os.path.splitext(filename)
    return 'visualizers/{name}_{version}.{ext}'.format(
        name=instance.visualizer.name.replace(' ', '_'),
        version=instance.visualizer.version.replace('.', '_'),
        ext=ext)


class VisualizerComponent(models.Model):
    visualizer = models.OneToOneField(
        Visualizer,
        on_delete=models.CASCADE,
        db_column='visualizer_id',
        verbose_name=_('visualizer'),
        help_text=_('Visualizer'),
        blank=False,
        null=False)
    javascript_file = models.FileField(
        upload_to=visualizer_path,
        db_column='javascript_file',
        verbose_name=_('javascript file'),
        help_text=_('Javascript file containing visualizer component'),
        blank=False,
        null=False)

    item_types = models.ManyToManyField(
        ItemType,
        through='VisualizerComponentItemType',
        through_fields=('visualizer_component', 'item_type'))

    class Meta:
        verbose_name = _('Visualizer Component')
        verbose_name_plural = _('Visualizers Components')

    def __str__(self):
        return str(self.visualizer)


class VisualizerComponentItemType(models.Model):
    item_type = models.ForeignKey(
        ItemType,
        on_delete=models.CASCADE,
        db_column='item_type_id',
        verbose_name=_('item type'),
        help_text=_('Item type'))
    visualizer_component = models.ForeignKey(
        'VisualizerComponent',
        on_delete=models.CASCADE,
        db_column='visualizer_component_id',
        verbose_name=_('visualizer component'),
        help_text=_('Visualizer component'))

    is_active = models.BooleanField(
        db_column='is_active',
        verbose_name=_('is active'),
        help_text=_('Is visualizer app active?'),
        default=True,
        blank=False,
        null=False)

    class Meta:
        verbose_name = _('Visualizer Component Item Type')
        verbose_name_plural = _('Visualizer Component Item Types')

        unique_together = (
            ('item_type', 'visualizer_component'),
        )

    def deactivate(self):
        self.is_active = False
        self.save()

    def save(self, *args, **kwargs):
        if self.is_active:
            queryset = VisualizerComponentItemType.objects.filter(
                item_type=self.item_type, is_active=True)
            for entry in queryset:
                if entry != self:
                    entry.deactivate()

        super().save(*args, **kwargs)



from django.contrib import admin
from selia_visualizers import models


@admin.register(models.VisualizerComponent, models.VisualizerComponentItemType)
class VisualizersAdmin(admin.ModelAdmin):
    pass



from django.views.decorators.http import require_http_methods
from django.http import JsonResponse

from selia_visualizers.utils import get_visualizer as get_visualizer_instance


@require_http_methods(["GET"])
def get_visualizer(request):
    item_type = request.GET.get('item_type', None)
    visualizer = get_visualizer_instance(item_type)

    response = {
        "visualizer": visualizer.visualizer_component.visualizer.pk,
        "file_url": visualizer.visualizer_component.javascript_file.url
    }
    return JsonResponse(response)



from django.conf.urls import url
from django.conf.urls import include
from django.conf import settings
from django.conf.urls.static import static


urlpatterns = [
    url(
        r'^visualizers/',
        include(('selia_visualizers.urls', 'selia_visualizers'))),
] + static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)



import os

from irekua_dev_settings.settings import *
from irekua_database.settings import *
from selia_visualizers.settings import *


BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
MEDIA_ROOT = os.path.join(os.path.dirname(BASE_DIR), 'media')
LOCALE_PATHS = [os.path.join(BASE_DIR, 'locale'), ]


INSTALLED_APPS = (
    SELIA_VISUALIZERS_APPS +
    IREKUA_DATABASE_APPS +
    IREKUA_BASE_APPS
)



from selia_visualizers.models import VisualizerComponentItemType


def get_visualizer(item_type):
    visualizer = VisualizerComponentItemType.objects.get(
        item_type=item_type,
        is_active=True)
    return visualizer



# Generated by Django 2.2.7 on 2019-11-25 06:52

from django.db import migrations, models
import django.db.models.deletion
import selia_visualizers.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('irekua_database', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='VisualizerComponent',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('javascript_file', models.FileField(db_column='javascript_file', help_text='Javascript file containing visualizer component', upload_to=selia_visualizers.models.visualizer_path, verbose_name='javascript file')),
            ],
            options={
                'verbose_name': 'Visualizer Component',
                'verbose_name_plural': 'Visualizers Components',
            },
        ),
        migrations.CreateModel(
            name='VisualizerComponentItemType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_active', models.BooleanField(db_column='is_active', default=True, help_text='Is visualizer app active?', verbose_name='is active')),
                ('item_type', models.ForeignKey(db_column='item_type_id', help_text='Item type', on_delete=django.db.models.deletion.CASCADE, to='irekua_database.ItemType', verbose_name='item type')),
                ('visualizer_component', models.ForeignKey(db_column='visualizer_component_id', help_text='Visualizer component', on_delete=django.db.models.deletion.CASCADE, to='selia_visualizers.VisualizerComponent', verbose_name='visualizer component')),
            ],
            options={
                'verbose_name': 'Visualizer Component Item Type',
                'verbose_name_plural': 'Visualizer Component Item Types',
                'unique_together': {('item_type', 'visualizer_component')},
            },
        ),
        migrations.AddField(
            model_name='visualizercomponent',
            name='item_types',
            field=models.ManyToManyField(through='selia_visualizers.VisualizerComponentItemType', to='irekua_database.ItemType'),
        ),
        migrations.AddField(
            model_name='visualizercomponent',
            name='visualizer',
            field=models.OneToOneField(db_column='visualizer_id', help_text='Visualizer', on_delete=django.db.models.deletion.CASCADE, to='irekua_database.Visualizer', verbose_name='visualizer'),
        ),
    ]



import os
from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), 'README.md')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))


version = {}
with open("selia_visualizers/version.py") as fp:
    exec(fp.read(), version)


setup(
    name='selia-visualizers',
    version='0.1',
    packages=find_packages(),
    include_package_data=True,
    license='BSD License',
    description='Visualizer provider for Selia',
    long_description=README,
    url='https://github.com/CONABIO-audio/selia-visualizers',
    author='CONABIO, Gustavo Everardo Robredo Esquivelzeta, Santiago Martínez Balvanera',
    author_email='erobredo@conabio.gob.mx, smartinez@conabio.gob.mx',
    install_requires=['irekua-database'],
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 2.1',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)
